@@ -21,8 +21,7 @@
 #  USA
 
 import os
-import gobject
-from gi.repository import Gtk
+from gi.repository import GObject, Gtk
 from gettext import gettext as _
 import threading
 import string
@@ -41,16 +40,21 @@ from softwareproperties.gtk.utils import *
 from softwareproperties.CountryInformation import CountryInformation
 
 def sort_mirrors(model, iter1, iter2, data=None):
-      ''' sort function for the mirror list:
+      """ sort function for the mirror list:
            - at first show all custom urls
            - secondly the separator
+           - show mirrors without a county first (e.g. the automatic mirror url)
            - third the official mirrors. if available
-             sort the countries '''
+             sort the countries
+      """
+
       #FIXME: cmp seems to prefer ASCI chars
-      (url1, sep1, custom1) = model.get(iter1, 0,1,2)
-      (url2, sep2, custom2) = model.get(iter2, 0,1,2)
+      (url1, sep1, custom1, mirror1) = model.get(iter1, 0, 1, 2, 3)
+      (url2, sep2, custom2, mirror2) = model.get(iter2, 0, 1, 2, 3)
+      has_child1 = model.iter_has_child(iter1)
+      has_child2 = model.iter_has_child(iter2)
       if custom1 and custom2:
-          return cmp(url1,url2)
+          return cmp(url1, url2)
       elif custom1:
           return -1
       elif custom2:
@@ -59,9 +63,12 @@ def sort_mirrors(model, iter1, iter2, data=None):
           return -1
       elif sep2:
           return 1
-      return cmp(url1,url2)
+      if has_child1 != has_child2:
+            return cmp(has_child1, has_child2)
+      return cmp(url1, url2)
 
 class DialogMirror:
+
   def __init__(self, parent, datadir, distro, custom_mirrors):
     """
     Initialize the dialog that allows to choose a custom or official mirror
@@ -92,20 +99,19 @@ class DialogMirror:
     self.label_action = self.label_test_mirror
 
     # store each proto and its dir
-    model_proto = Gtk.ListStore(gobject.TYPE_STRING,
-                                gobject.TYPE_STRING)
+    model_proto = Gtk.ListStore(GObject.TYPE_STRING,
+                                GObject.TYPE_STRING)
     self.combobox.set_model(model_proto)
     cr = Gtk.CellRendererText()
     self.combobox.pack_start(cr, True)
     self.combobox.add_attribute(cr, "markup", 0)
 
-    self.model = Gtk.TreeStore(gobject.TYPE_STRING,  # COLUMN_URI
-                               gobject.TYPE_BOOLEAN, # COLUMN_SEPARATOR
-                               gobject.TYPE_BOOLEAN, # COLUMN_CUSTOM
-                               gobject.TYPE_PYOBJECT)# COLUMN_MIRROR
+    self.model = Gtk.TreeStore(GObject.TYPE_STRING,  # COLUMN_URI
+                               GObject.TYPE_BOOLEAN, # COLUMN_SEPARATOR
+                               GObject.TYPE_BOOLEAN, # COLUMN_CUSTOM
+                               GObject.TYPE_PYOBJECT)# COLUMN_MIRROR
     self.treeview.set_row_separator_func(is_separator, None)
     self.model_sort = Gtk.TreeModelSort(model=self.model)
-    self.model_sort.set_default_sort_func(sort_mirrors, None)
 
     self.distro = distro
 
@@ -147,13 +153,15 @@ class DialogMirror:
             model.append(parent, [hostname, False, False, mirror]),
             map_loc[mirror.location] = parent
         else:
-            model.append(None, [hostname, False, False, mirror])
+            it = model.append(None, [hostname, False, False, mirror])
     # Scroll to the local mirror set
     if patriot != None:
         path_sort = self.model_sort.get_path(self.model_sort.convert_child_iter_to_iter(patriot)[1])
         self.treeview.expand_row(path_sort, False)
         self.treeview.set_cursor(path_sort, None, False)
         self.treeview.scroll_to_cell(path_sort, use_align=True, row_align=0.5)
+    # set the sort function, this will also trigger a sort
+    self.model_sort.set_default_sort_func(sort_mirrors, None)
 
   def on_edited_custom_mirror(self, cell, path, new_text, model):
     ''' Check if the new mirror uri is faild, if yes change it, if not
