@@ -24,15 +24,12 @@
 
 import apt
 import apt_pkg
+import dbus
 import tempfile
 from gettext import gettext as _
 import os
-import re
-from xml.sax.saxutils import escape
 
-from gi.repository import Gdk
-from gi.repository import Gtk
-import gobject
+from gi.repository import GObject, Gdk, Gtk, Gio
 
 from SimpleGtkbuilderApp import SimpleGtkbuilderApp
 from aptsources.sourceslist import SourceEntry
@@ -68,18 +65,19 @@ RESPONSE_ADD = 2
 ) = range(5)
 
 def error(parent_window, summary, msg):
-  " show a error dialog "
-  dialog = Gtk.MessageDialog(parent=parent_window,
-                             flags=Gtk.DialogFlags.MODAL,
-                             type=Gtk.MessageType.ERROR,
-                             buttons=Gtk.ButtonsType.OK,
-                             message_format=None)
-  dialog.set_markup("<big><b>%s</b></big>\n\n%s" % (summary, msg))
-  res = dialog.run()
-  dialog.destroy()
-  return False
+    """ show a error dialog """
+    dialog = Gtk.MessageDialog(parent=parent_window,
+                               flags=Gtk.DialogFlags.MODAL,
+                               type=Gtk.MessageType.ERROR,
+                               buttons=Gtk.ButtonsType.OK,
+                               message_format=None)
+    dialog.set_markup("<big><b>%s</b></big>\n\n%s" % (summary, msg))
+    res = dialog.run()
+    dialog.destroy()
+    return False
 
 class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
+
   def __init__(self, datadir=None, options=None, file=None, parent=None):
     """ Provide a GTK based graphical user interface to configure
         the used software repositories, corresponding authentication keys
@@ -87,8 +85,9 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     SoftwareProperties.__init__(self, options=options, datadir=datadir)
     Gtk.Window.set_default_icon_name("software-properties")
 
-    SimpleGtkbuilderApp.__init__(self, os.path.join(datadir, "gtkbuilder", "main.ui"),
-                            domain="software-properties")
+    SimpleGtkbuilderApp.__init__(self,
+                                 os.path.join(datadir, "gtkbuilder", "main.ui"),
+                                 domain="software-properties")
 
     if parent:
       self.window_main.set_type_hint(Gdk.WindowTypeHint.DIALOG)
@@ -115,6 +114,32 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     if options and options.open_tab:
       self.notebook_main.set_current_page(int(options.open_tab))
 
+    # gsettings
+    all_schemas = Gio.Settings.list_schemas()
+    if "com.ubuntu.update-notifier" in all_schemas:
+        self.settings = Gio.Settings("com.ubuntu.update-notifier")
+        # we need this for reverting
+        self.initial_auto_launch = self.settings.get_int("regular-auto-launch-interval") 
+    else:
+        self.settings = None
+        self.initial_auto_launch = 0
+        self.combobox_other_updates.set_sensitive(False)
+
+    # get the dbus backend
+    bus = dbus.SystemBus()
+    proxy = bus.get_object("com.ubuntu.SoftwareProperties", "/")
+    self.backend = dbus.Interface(proxy, "com.ubuntu.SoftwareProperties")
+    self.backend.connect_to_signal(
+        "SourcesListModified", self.on_sources_list_modified)
+    self.backend.connect_to_signal(
+        "ConfigModified", self.on_config_modified)
+    self.backend.connect_to_signal(
+        "KeysModified", self.on_keys_modified)
+    self.backend.connect_to_signal(
+        "AuthFailed", self.on_auth_failed)
+    self.backend.connect_to_signal(
+        "CdromScanFailed", self.on_cdrom_scan_failed)
+    
     # Show what we have early
     self.window_main.show()
 
@@ -144,13 +169,13 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     #       We have got some bug reports from users calling
     #       "sudo software-properties-gtk /etc/apt/sources.list" from the
     #       command line.
-    if file != None and \
+    if (file != None and
        os.path.abspath(file) !=  "%s%s" % (apt_pkg.config.find_dir("Dir::Etc"),
-                                           apt_pkg.config.find("Dir::Etc::sourcelist")):
+                                           apt_pkg.config.find("Dir::Etc::sourcelist"))):
         self.open_file(file)
 
   def update_interface(self):
-    " abstract interface to keep the UI alive "
+    """ abstract interface to keep the UI alive """
     while Gtk.events_pending():
       Gtk.main_iteration()
 
@@ -159,7 +184,8 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
         corresponding to the used distro """
     is_helpful = self.get_popcon_participation()
     if is_helpful != None:
-      self.label_popcon_desc.set_label(softwareproperties.distro.get_popcon_description(self.distro))
+      self.label_popcon_desc.set_label(
+          softwareproperties.distro.get_popcon_description(self.distro))
       self.vbox_popcon.show()
       self.checkbutton_popcon.set_active(is_helpful)
 
@@ -167,30 +193,24 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     " setup the widgets that allow configuring the release upgrades "
     i = self.get_release_upgrades_policy()
     self.combobox_release_upgrades.set_active(i)
-    self.combobox_release_upgrades.connect('changed', self.on_combobox_release_upgrades_changed)
+    self.combobox_release_upgrades.connect(
+        'changed', self.on_combobox_release_upgrades_changed)
 
   def init_auto_update(self):
     """ Set up the widgets that allow to configure the update automation """
-    self.combobox_update_interval = Gtk.ComboBoxText.new_with_entry()
-    self.hbox_check_for_updates.pack_start(self.combobox_update_interval, True, True, 0)
     self.combobox_update_interval.show()
-    
+
+    # normal updates
     # this maps the key (combo-box-index) to the auto-update-interval value
-    # where (-1) means, no key
-    self.combobox_interval_mapping = { 0 : 1,
-                                       1 : 2,
-                                       2 : 7,
-                                       3 : 14 }
-    self.combobox_update_interval.set_active(0)
+    # we build it dynamically from the model
+    model = self.combobox_update_interval.get_model()
+    self.combobox_interval_mapping = {}
+    for (i, row) in enumerate(model):
+      # column 1 is the update interval in days
+      value = model.get_value(row.iter, 1)
+      self.combobox_interval_mapping[i] = value
 
-    #update_days = apt_pkg.Config.FindI(softwareproperties.CONF_MAP["autoupdate"])
-    self.combobox_update_interval.append_text(_("Daily"))
-    self.combobox_update_interval.append_text(_("Every two days"))
-    self.combobox_update_interval.append_text(_("Weekly"))
-    self.combobox_update_interval.append_text(_("Every two weeks"))
-
-    model_check_interval = Gtk.ListStore(gobject.TYPE_STRING,
-                                         gobject.TYPE_INT)
+    # normal updates
     update_days = self.get_update_interval()
 
     # If a custom period is defined add a corresponding entry
@@ -198,78 +218,62 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
         if update_days > 0:
             self.combobox_update_interval.append_text(_("Every %s days") 
                                                       % update_days)
-            self.combobox_interval_mapping[4] = update_days
+            self.combobox_interval_mapping[-1] = update_days
     
     for key in self.combobox_interval_mapping:
       if self.combobox_interval_mapping[key] == update_days:
         self.combobox_update_interval.set_active(key)
         break
 
-    if update_days >= 1:
-      self.checkbutton_auto_update.set_active(True)
-      self.combobox_update_interval.set_sensitive(True)
-      self.vbox_auto_updates.set_sensitive(True)
-    else:
-      self.checkbutton_auto_update.set_active(False)
-      self.combobox_update_interval.set_sensitive(False)
-      self.vbox_auto_updates.set_sensitive(False)
-
-    self.handlers.append(
-        (self.checkbutton_auto_update,
-         self.checkbutton_auto_update.connect("toggled", 
-                                     self.on_auto_update_toggled)))
     self.handlers.append(
         (self.combobox_update_interval,
          self.combobox_update_interval.connect("changed", 
                                      self.on_combobox_update_interval_changed)))
+    
     self.handlers.append(
-        (self.radiobutton_updates_download,
-         self.radiobutton_updates_download.connect("toggled", 
-                                     self.set_update_automation_level,
-                                     softwareproperties.UPDATE_DOWNLOAD)))
+        (self.combobox_security_updates,
+         self.combobox_security_updates.connect("changed",
+                                     self.set_sec_update_automation_level)))
+    
     self.handlers.append(
-        (self.radiobutton_updates_inst_sec,
-         self.radiobutton_updates_inst_sec.connect("toggled", 
-                                     self.set_update_automation_level,
-                                     softwareproperties.UPDATE_INST_SEC)))
-    self.handlers.append(
-        (self.radiobutton_updates_notify,
-         self.radiobutton_updates_notify.connect("toggled", 
-                                     self.set_update_automation_level,
-                                     softwareproperties.UPDATE_NOTIFY)))
-
+        (self.combobox_security_updates,
+         self.combobox_other_updates.connect("changed",
+                                     self.set_other_update_automation_level)))
+  
   def show_auto_update_level(self):
     """Represent the level of update automation in the user interface"""
-    level = self.get_update_automation_level()
-    self.block_handlers()
-    if level == None:
-        self.radiobutton_updates_inst_sec.set_inconsistent(True)
-        self.radiobutton_updates_download.set_inconsistent(True)
-        self.radiobutton_updates_notify.set_inconsistent(True)
+
+    # Security Updates
+    level_sec = self.get_update_automation_level()
+    if level_sec == None:
+      self.combobox_security_updates.set_sensitive(False)
     else:
-        self.radiobutton_updates_inst_sec.set_inconsistent(False)
-        self.radiobutton_updates_download.set_inconsistent(False)
-        self.radiobutton_updates_notify.set_inconsistent(False)
-    if level == softwareproperties.UPDATE_MANUAL or \
-       level == softwareproperties.UPDATE_NOTIFY:
-        self.radiobutton_updates_notify.set_active(True)
-    elif level == softwareproperties.UPDATE_DOWNLOAD:
-        self.radiobutton_updates_download.set_active(True)
-    elif level == softwareproperties.UPDATE_INST_SEC:
-        self.radiobutton_updates_inst_sec.set_active(True)
-    # Unblock the toggle handlers
-    self.unblock_handlers()
+      self.combobox_security_updates.set_sensitive(True)
+          
+    if (level_sec == softwareproperties.UPDATE_MANUAL or
+        level_sec == softwareproperties.UPDATE_NOTIFY):
+      self.combobox_security_updates.set_active(0) # Display immediately
+    elif level_sec == softwareproperties.UPDATE_DOWNLOAD:
+      self.combobox_security_updates.set_active(1) # Download automatically
+    elif level_sec == softwareproperties.UPDATE_INST_SEC:
+      self.combobox_security_updates.set_active(2) # Download and install automatically
+    
+    # Other Updates
+    if self.settings:
+        level_other = self.settings.get_int("regular-auto-launch-interval")
+        model = self.combobox_other_updates.get_model()
+        for (i, row) in enumerate(model):
+            level = model.get_value(row.iter, 1)
+            if level_other == level:
+                self.combobox_other_updates.set_active(i)
+                break
 
   def init_distro(self):
     """Setup the user interface elements to represent the distro"""
 
-    # TRANS: %s stands for the distribution name e.g. Debian or Ubuntu
-    self.label_updates.set_label("<b>%s</b>" % (_("%s updates") %\
-                                                self.distro.id.encode('UTF-8')))
     # TRANS: %s stands for the distribution name e.g. Debian or Ubuntu
     self.label_dist_name.set_label(_("%s Software") % self.distro.id.encode('UTF-8'))
 
-
     self.handlers.append((self.checkbutton_source_code,
                           self.checkbutton_source_code.connect("toggled",
                               self.on_checkbutton_source_code_toggled)))
@@ -321,9 +325,9 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     self.handlers.append((self.combobox_server,
                           self.combobox_server.connect("changed",
                               self.on_combobox_server_changed)))
-    server_store = Gtk.ListStore(gobject.TYPE_STRING,
-                                 gobject.TYPE_STRING,
-                                 gobject.TYPE_BOOLEAN)
+    server_store = Gtk.ListStore(GObject.TYPE_STRING,
+                                 GObject.TYPE_STRING,
+                                 GObject.TYPE_BOOLEAN)
     self.combobox_server.set_model(server_store)
     self.combobox_server.set_row_separator_func(self.is_row_separator, 2)
 
@@ -338,6 +342,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
   def show_distro(self):
     """Fill the distro user interface with life"""
     self.block_handlers()
+
     # Enable or disable the child source checkbuttons
     for checkbox in self.vbox_updates.get_children():
         (active, inconsistent) = self.get_comp_child_state(checkbox.template)
@@ -425,13 +430,28 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
         for source in self.sourceslist_visible:
             self.print_source_entry(source)
 
-  def set_update_automation_level(self, widget, state):
-    '''Call the backend to set the update automation level to the given 
-       value'''
-    if widget.get_active() == True:
-        self.vbox_auto_updates.foreach(lambda b, d: b.set_inconsistent(False), None)
-        SoftwareProperties.set_update_automation_level(self, state)
-    self.set_modified_config()
+  def set_sec_update_automation_level(self, widget):
+    """Call the backend to set the security update automation level to the given
+       value"""
+    index = widget.get_active()
+    state = -1
+    if index == 0: # Display immediately
+      state = softwareproperties.UPDATE_NOTIFY
+    elif index == 1: # Download automatically
+      state = softwareproperties.UPDATE_DOWNLOAD
+    elif index == 2: # Download and install automatically
+      state = softwareproperties.UPDATE_INST_SEC
+    # only set if something actually changed
+    if state != self.get_update_automation_level():
+        self.backend.SetUpdateAutomationLevel(state)
+    
+  def set_other_update_automation_level(self, widget):
+    """Set the other update automation level to the given value via gconf"""
+    index = widget.get_active()
+    model = self.combobox_other_updates.get_model()
+    # the second column is the update interval days
+    days = model[index][1]
+    self.settings.set_int("regular-auto-launch-interval", days)
 
   def is_row_separator(self, model, iter, column=0):
     ''' Check if a given row is a separator '''
@@ -441,7 +461,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     """ set the release upgrades policy """
     #print "on_combobox_release_upgrades_changed()"
     i = combobox.get_active()
-    self.set_release_upgrades_policy(i)
+    self.backend.SetReleaseUpgradesPolicy(i)
 
   def on_combobox_server_changed(self, combobox):
     """
@@ -461,17 +481,15 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
                               self.custom_mirrors)
         res = dialog.run()
         if res != None:
-            self.distro.change_server(res)
-            self.set_modified_sourceslist()
+            self.backend.ChangeMainDownloadServer(res)
         else:
             combobox.set_active(self.active_server)
     elif uri != None and len(self.distro.used_servers) > 0:
         self.active_server = combobox.get_active()
-        self.distro.change_server(uri)
-        self.distro.default_server = uri
-        self.set_modified_sourceslist()
-    else:
-        self.distro.default_server = uri
+        self.backend.ChangeMainDownloadServer(uri)
+    # mvo: is this still needed?
+    #else:
+    #    self.distro.default_server = uri
 
   def on_component_toggled(self, checkbutton, comp):
     """
@@ -479,30 +497,32 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     child sources and source code sources
     """
     if checkbutton.get_active() == True:
-        self.enable_component(comp)
+        self.backend.EnableComponent(comp)
     else:
-        self.disable_component(comp)
-    self.set_modified_sourceslist()
+        self.backend.DisableComponent(comp)
 
   def on_checkbutton_child_toggled(self, checkbutton, template):
     """
     Enable or disable a child repo of the distribution main repository
     """
     if checkbutton.get_active() == False:
-        self.disable_child_source(template)
+        self.backend.DisableChildSource(template.name)
     else:
-        self.enable_child_source(template)
+        self.backend.EnableChildSource(template.name)
           
   def on_checkbutton_source_code_toggled(self, checkbutton):
     """ Disable or enable the source code for all sources """
     if checkbutton.get_active() == True:
-        self.enable_source_code_sources()
+        self.backend.EnableSourceCodeSources()
     else:
-        self.disable_source_code_sources()
+        self.backend.DisableSourceCodeSources()
 
   def on_checkbutton_popcon_toggled(self, widget):
     """ The user clicked on the popcon paritipcation button """
-    self.set_popcon_pariticipation(widget.get_active())
+    # only trigger the backend if something actually changed
+    do_popcon = self.get_popcon_participation()
+    if widget.get_active() != do_popcon:
+        self.backend.SetPopconPariticipation(widget.get_active())
 
   def open_file(self, file):
     """Show a confirmation for adding the channels of the specified file"""
@@ -517,12 +537,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
         self.sourceslist.list = []
     if res in (RESPONSE_ADD, RESPONSE_REPLACE):
         for source in new_sources:
-            self.sourceslist.add(source.type,
-                                 source.uri,
-                                 source.dist,
-                                 source.comps,
-                                 source.comment)
-        self.set_modified_sourceslist()
+            self.backend.AddSourceFromLine(str(source))
 
   def on_sources_drag_data_received(self, widget, context, x, y,
                                      selection, target_type, timestamp):
@@ -544,17 +559,17 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     # STORE_SOURCE - the source entry object
     # STORE_SEPARATOR - if the entry is a separator
     # STORE_VISIBLE - if the entry is shown or hidden
-    self.cdrom_store = Gtk.ListStore(gobject.TYPE_BOOLEAN, 
-                                     gobject.TYPE_STRING,
-                                     gobject.TYPE_PYOBJECT,
-                                     gobject.TYPE_BOOLEAN,
-                                     gobject.TYPE_BOOLEAN)
+    self.cdrom_store = Gtk.ListStore(GObject.TYPE_BOOLEAN, 
+                                     GObject.TYPE_STRING,
+                                     GObject.TYPE_PYOBJECT,
+                                     GObject.TYPE_BOOLEAN,
+                                     GObject.TYPE_BOOLEAN)
     self.treeview_cdroms.set_model(self.cdrom_store)
-    self.source_store = Gtk.ListStore(gobject.TYPE_BOOLEAN, 
-                                      gobject.TYPE_STRING,
-                                      gobject.TYPE_PYOBJECT,
-                                      gobject.TYPE_BOOLEAN,
-                                      gobject.TYPE_BOOLEAN)
+    self.source_store = Gtk.ListStore(GObject.TYPE_BOOLEAN, 
+                                      GObject.TYPE_STRING,
+                                      GObject.TYPE_PYOBJECT,
+                                      GObject.TYPE_BOOLEAN,
+                                      GObject.TYPE_BOOLEAN)
     self.treeview_sources.set_model(self.source_store)
     self.treeview_sources.set_row_separator_func(self.is_separator,
                                                  STORE_SEPARATOR)
@@ -629,7 +644,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     #FIXME cdroms need to disable the comps in the childs and sources
     iter = store.get_iter((int(path),))
     source_entry = store.get_value(iter, STORE_SOURCE) 
-    self.toggle_source_use(source_entry)
+    self.backend.ToggleSourceUse(str(source_entry))
 
   def init_keys(self):
     """Setup the user interface parts needed for the key handling"""
@@ -682,28 +697,46 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
                 "or it might be corrupt."))
       self.show_keys()
 
-  #FIXME revert automation settings too
   def on_button_revert_clicked(self, button):
-    """Restore the source list from the startup of the dialog"""
-    SoftwareProperties.revert(self)
-    self.set_modified_sourceslist()
-    self.show_auto_update_level()
-    self.button_revert.set_sensitive(False)
-    self.modified_sourceslist = False
+      """Restore the source list from the startup of the dialog"""
+      self.backend.Revert()
+      if self.settings:
+          self.settings.set_int("regular-auto-launch-interval", self.initial_auto_launch)
+      self.show_auto_update_level()
+      self.button_revert.set_sensitive(False)
+      self.modified_sourceslist = False
 
-  def set_modified_config(self):
+  # dbus events
+  def on_config_modified(self):
     """The config was changed and now needs to be saved and reloaded"""
-    SoftwareProperties.set_modified_config(self)
+    apt.apt_pkg.init_config()
     self.button_revert.set_sensitive(True)
 
-  def set_modified_sourceslist(self):
+  def on_keys_modified(self):
+    """ The apt keys have changed and need to be redisplayed """
+    self.show_keys()
+
+  def on_sources_list_modified(self):
     """The sources list was changed and now needs to be saved and reloaded"""
-    SoftwareProperties.set_modified_sourceslist(self)
+    self.reload_sourceslist()
     self.show_distro()
     self.show_isv_sources()
     self.show_cdrom_sources()
     self.button_revert.set_sensitive(True)
 
+  def on_auth_failed(self):
+    """ send when the authentication failed """
+    # reread the current config
+    self.on_sources_list_modified()
+    self.on_config_modified()
+    self.on_keys_modified()
+
+  def on_cdrom_scan_failed(self):
+      error(self.window_main,
+            _("Error scanning the CD"),
+            _("Could not find a suitable CD."))
+
+  # helpers
   def show_isv_sources(self):
     """ Show the repositories of independent software vendors in the
         third-party software tree view """
@@ -746,28 +779,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     i = self.combobox_update_interval.get_active()
     if i != -1:
         value = self.combobox_interval_mapping[i]
-        self.set_update_interval(value)
-
-  def on_auto_update_toggled(self, widget):
-    """Enable or disable automatic updates and modify the user interface
-       accordingly"""
-    if self.checkbutton_auto_update.get_active():
-      self.combobox_update_interval.set_sensitive(True)
-      self.vbox_auto_updates.set_sensitive(True)
-      # if no frequency was specified use daily
-      i = self.combobox_update_interval.get_active()
-      if i == -1:
-          i = 0
-          self.combobox_update_interval.set_active(i)
-      value = self.combobox_interval_mapping[i]
-      # A little hack to re-set the former selected update automation level
-      self.vbox_auto_updates.foreach(lambda b, d: b.toggled(), None)
-    else:
-      self.combobox_update_interval.set_sensitive(False)
-      self.vbox_auto_updates.set_sensitive(False)
-      SoftwareProperties.set_update_automation_level(self, None)
-      value = 0
-    self.set_update_interval(str(value))
+        self.backend.SetUpdateInterval(value)
 
   def on_add_clicked(self, widget):
     """Show a dialog that allows to enter the apt line of a to be used repo"""
@@ -775,8 +787,7 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
                        self.datadir, self.distro)
     line = dialog.run()
     if line != None:
-      self.add_source_from_line(line)
-      self.set_modified_sourceslist()
+      self.backend.AddSourceFromLine(line)
 
   def on_edit_clicked(self, widget):
     """Show a dialog to edit an ISV source"""
@@ -784,11 +795,12 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     (model, iter) = sel.get_selected()
     if not iter:
       return
-    source_entry = model.get_value(iter, LIST_ENTRY_OBJ)
+    old_source_entry = model.get_value(iter, LIST_ENTRY_OBJ)
     dialog = DialogEdit(self.window_main, self.sourceslist,
-                        source_entry, self.datadir)
+                        old_source_entry, self.datadir)
     if dialog.run() == Gtk.ResponseType.OK:
-        self.set_modified_sourceslist()
+        self.backend.ReplaceSourceEntry(str(old_source_entry), 
+                                        str(dialog.new_source_entry))
 
   # FIXME:outstanding from merge
   def on_isv_source_activated(self, treeview, path, column):
@@ -823,8 +835,8 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     (path, column) = self.treeview_sources.get_cursor()
     iter = model.get_iter(path)
     if iter:
-      self.remove_source(model.get_value(iter, LIST_ENTRY_OBJ))
-      self.set_modified_sourceslist()
+        source_entry = model.get_value(iter, LIST_ENTRY_OBJ)
+        self.backend.RemoveSource(str(source_entry))
 
   def add_key_clicked(self, widget):
     """Provide a file chooser that allows to add the gnupg of a trusted
@@ -840,12 +852,11 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     res = chooser.run()
     chooser.hide()
     if res == Gtk.ResponseType.ACCEPT:
-      if not self.add_key(chooser.get_filename()):
+      if not self.backend.AddKey(chooser.get_filename()):
         error(self.window_main,
               _("Error importing selected file"),
               _("The selected file may not be a GPG key file " \
                 "or it might be corrupt."))
-      self.show_keys()
 
   def remove_key_clicked(self, widget):
     """Remove a trusted software vendor key"""
@@ -854,17 +865,15 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     if a_iter == None:
         return
     key = model.get_value(a_iter,0)
-    if not self.remove_key(key[:8]):
+    if not self.backend.RemoveKey(key[:8]):
       error(self.main,
         _("Error removing the key"),
         _("The key you selected could not be removed. "
           "Please report this as a bug."))
-    self.show_keys()
 
   def on_restore_clicked(self, widget):
     """Restore the original keys"""
-    self.apt_key.update()
-    self.show_keys()
+    self.backend.UpdateKeys()
 
   def on_delete_event(self, widget, args):
     """Close the window if requested"""
@@ -881,34 +890,5 @@ class SoftwarePropertiesGtk(SoftwareProperties,SimpleGtkbuilderApp):
     self.quit()
 
   def on_button_add_cdrom_clicked(self, widget):
-    '''Show a dialog that allows to add a repository located on a CDROM
-       or DVD'''
-    # testing
-    #apt_pkg.Config.Set("APT::CDROM::Rename","true")
-
-    saved_entry = apt_pkg.config.find("Dir::Etc::sourcelist")
-    tmp = tempfile.NamedTemporaryFile()
-    apt_pkg.config.set("Dir::Etc::sourcelist",tmp.name)
-    progress = CdromProgress(self.datadir,self.window_main)
-    cdrom = apt_pkg.Cdrom()
-    # if nothing was found just return
-    try:
-      res = cdrom.add(progress)
-    except SystemError, msg:
-      error(self.window_main, _("Error scanning the CD"), msg)
-      return
-    finally:
-      apt_pkg.config.set("Dir::Etc::sourcelist",saved_entry)
-      progress.dialog_cdrom_progress.hide()
-
-    if res == False:
-      return
-    # read tmp file with source name (read only last line)
-    line = ""
-    for x in open(tmp.name):
-      line = x
-    if line != "":
-      full_path = "%s%s" % (apt_pkg.config.find_dir("Dir::Etc"),saved_entry)
-      # insert cdrom source first, so that it has precedence over network sources
-      self.sourceslist.list.insert(0, SourceEntry(line,full_path))
-      self.set_modified_sourceslist()
+      """ when a cdrom is requested for adding """
+      self.backend.AddCdromSource()
